// @import "actiontext";

label.required:after {
  content: " *";
  color: red;
}

.fragment {
  @extend .border;
  @extend .p-3;
}

form, .alert, .fragment {
  > :last-child {
    margin-bottom: 0 !important;
  }
}
